<?php
/*
Plugin Name: Allow Categories
Plugin URI: http://jameslow.com
Description: Per-category permissions for Wordpress. English + Allow version of L. Fargue http://stknights.free.fr/?p=69
Version: 0.4.3
Author: James Low
Author URI: http://jameslow.com
*/

/*  
    Copyright 2007  James LOW

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


class Allow_Category{

	function allow_getAllCategories() {
		return get_categories(array('hide_empty' => false));
	}

	function allow_getCategory() {
		global $user_ID;
		$user = Allow_Category::allow_getUser($user_ID);
		$public = Allow_Category::allow_getPublic();
		if ($user && $public) {
			return array_merge($user,$public);
		} elseif ($user) {
			return $user;
		} else {
			return $public;
		}
	}

	function allow_getUser($user) {
		$opts = get_option("allow_option");
		$keys = array_keys($opts);
		$sid = (string)$user;
		if (in_array($sid, $keys)) {
			return $opts[$sid];
		}
		return false;
	}

	function allow_getPublic() {
		return Allow_Category::allow_getUser("public");
	}
	
	function allow_removeCategorySelection($page) {
		global $catUserPreg;
		$cats = Allow_Category::allow_getAllCategories();
		foreach ($cats as $cat){
			$keep = false;
			if ($catUserPreg) {
				//might need to add code here, to make sure old categories remain, even if user doesn't have permission
				if (in_array($cat->cat_ID,$catUserPreg)) {
					$keep = true;
				}
			}
			if (!$keep) {
				if ( count(get_categories(array('child_of' => $cat->cat_ID))) ) {
					$page=preg_replace("#<li id=\"category-$cat->cat_ID\".*?</ul>#sim", "", $page);
				}else{
					$page=preg_replace("#<li id=\"category-$cat->cat_ID\".*?</li>#sim", "", $page);
				}
			}
		}
		return $page;
	}

	function allow_wpversion() {
		global  $wp_version;
		$version = explode (".", $wp_version);
		return ($version[0] > 2 || ($version[0] == 2 && $version[1] >= 3));
	}

	function allow_where( $where )
	{	
		global $userdata;
		global $wpdb;
		
		if ($userdata->user_level < 10) {
			$catUser = Allow_Category::allow_getCategory();

			if (Allow_Category:: allow_wpversion()) {
				$ids = get_objects_in_term($catUser, 'category');
				if ( is_wp_error( $ids ) ) {
					return $ids;
				}

			} else {
				$query = 'SELECT p.ID FROM wp_posts p, wp_post2cat p2c, wp_categories c WHERE p.ID = p2c.post_id AND p2c.category_id = c.cat_ID';
				$query .= ' AND category_id in (' . implode(', ', $catUser) . ')';
				$ids = $wpdb->get_col($query . ';');
			}
			if ( is_array($ids) && count($ids > 0) ) {
				$out_posts = implode(', ', $ids);
				$where .= " AND $wpdb->posts.ID IN ($out_posts)";
			}
		}
		return $where;
	}

	function allow_notAllowed($page){
		return preg_replace("#<form name=\"post\".*?</form>#sim", "<div class=\"error\"><p><strong>You are not authorised to edit posts in this category.</strong><br />
				<a href=\"".$_SERVER['HTTP_REFERER']."\">Return to previous page.</a></p></div>", $page);
	}

	function allow_adminHead($in) {
		global $userdata;
		global $catUserPreg;
		get_currentuserinfo();
		$catUserPreg = Allow_Category::allow_getCategory();

		if ($userdata->user_level < 10) {
			if(preg_match("#/wp-admin/post-new.php#", $_SERVER['REQUEST_URI']) ) {
				ob_start(array('Allow_Category', 'allow_removeCategorySelection'));
			}
			if(preg_match("#/wp-admin/post.php#", $_SERVER['REQUEST_URI']) ) {
					$trouve=false;
					if ($catUserPreg) {
						$theCat=get_the_category();
						foreach($theCat as $cat_u){
							if (in_array($cat_u->cat_ID,$catUserPreg)){
								$trouve=true;
								break;
							}
						}
					}
					if ($trouve){
						ob_start(array('Allow_Category', 'allow_removeCategorySelection'));
					}else{
						ob_start(array('Allow_Category', 'allow_notAllowed'));
					}
			}
		}
		return $in;
	}
	
	function allow_delPost($in){
		global $wpdb;
		$article=get_post($in);
		$catUser = Allow_Category::allow_getCategory();
		if ($userdata->user_level < 10) {
			$trouve=false;
			if ($catUser) {
				$cats = get_the_category($in);

				foreach($cats as $ct){
					if (in_array($ct->cat_ID,$catUser)) {
						$trouve=true;
						break;
					}
				}
			}
			if (!$trouve) {
				die("You are not autorised to delete posts in this category.");
			}
		}
	}
	
	function removeArrayElement(&$arr, $index) {
		if(isset($arr[$index])){
        		array_splice($arr, $index, 1);
   		 }
	}

	function allow_catRedirect() {
		global $userdata;
		get_currentuserinfo();
		$force_login = get_option("allow_force_login");
		if (!$userdata->user_login && $force_login) {
			//if ( (!empty($_COOKIE[USER_COOKIE]) && !wp_login($_COOKIE[USER_COOKIE], $_COOKIE[PASS_COOKIE], true)) || (empty($_COOKIE[USER_COOKIE])) ) {
			nocache_headers();
			header("HTTP/1.1 302 Moved Temporarily");
			header('Location: ' . get_settings('siteurl') . '/wp-login.php?redirect_to=' . urlencode($_SERVER['REQUEST_URI']));
            		header("Status: 302 Moved Temporarily");
			exit();
			//}
		}

		/*
		if ($userdata->user_login){
			$catUser = Allow_Category::allow_getCategory();
			if ($catUser && $userdata->user_level < 10) {
					$theCat=get_the_category();				

					foreach($theCat as $cat_u) {
						echo $cat_u->term_id . ",";
						if (in_array($cat_u->term_id,$catUser)){
							header("HTTP/1.0");
							echo "
							<script language=\"javascript\">\n
							//alert('You are not authorised to view posts in this category.');
							//document.location='".$_SERVER['HTTP_REFERER']."';
							</script>";
							//die();
						}
					}
			}
		}
		*/
	}
	
	function allow_form() {
		global $wpdb;
		if (isset($_POST['info_update'])) {
			$updated = Allow_Category::allow_saveForm($_POST);
			if ($updated) {
				echo "<div class=\"updated\"><p><strong>" . __('Settings saved.', 'allow_option') ."</strong></p></div>";
			} else {
				echo "<div class=\"error\"><p><strong>" . __('There was an error while saving.', 'allow_option') ."</strong></p></div>";			
			}
		}
		echo "<div class=\"wrap\"><form method=\"post\" action=\"\">";
		echo "<h2>Allow access to category parents.</h2>";
		echo "<p>Check the category to ALLOW permission.</p>";

		if (function_exists('get_authors')) {
			$wpusers = get_authors();
			$userIds = array();
			foreach ($wpusers as $wpuser) {
				//TODO: When wordpress implement get_autors, check this works
				$userIds[] = $wpuser->ID;
			}
		} else {
			$userIds = $wpdb->get_col("SELECT ID FROM $wpdb->users;");
		}
		$users = array();
		$cats = Allow_Category::allow_getAllCategories();
		
		$opts = get_option("allow_option");
	
		echo "<table class=\"widefat\">
		<thead>
		<tr>
		<th scope=\"col\"><div style=\"text-align: center\">User</div></th>";
		foreach ($cats as $ct){
			echo "<th scope=\"col\"><div style=\"text-align: center\">$ct->cat_name</div></th>";
		}
		echo "</tr>
		</thead>";

		$i=0;
		$userId = "public";
		echo '<tr><td>(Public)';
		echo '<input type="hidden" name="user[]" value="'. $userId.'" />';
		echo '</td>';
		foreach ($cats as $ct){
			echo "<td><div align=\"center\"><input name=\"cat[$i][]\" type=\"checkbox\" value=\"$ct->cat_ID\"";
			if ($opts["$userId"] && in_array($ct->cat_ID,$opts["$userId"])) echo "checked";
			echo "/></div></td>";
		}
		echo '</tr>';

		$class1="alternate author-other status-publish";
		$class2="author-other status-publish";
		$i=1;
		foreach ($userIds as $userId) {
			$tmp_user = new WP_User($userId);
			if ($tmp_user->wp_user_level > 7) continue;
			$users[$userId] = $tmp_user;
			echo "<tr class=\"$class1\"><td>$tmp_user->user_nicename<input type=\"hidden\" name=\"user[]\" value=\"$tmp_user->ID\" /></td>";
			foreach($cats as $ct){
				echo "<td><div align=\"center\"><input name=\"cat[$i][]\" type=\"checkbox\" value=\"$ct->cat_ID\"";
				if ($opts["$userId"] && in_array($ct->cat_ID,$opts["$userId"])) echo "checked";
				echo "/></div></td>";
			}
			echo "</tr>\n";
			$classTemp=$class1;
			$class1=$class2;
			$class2=$classTemp;
			$i++;
		}
	
		echo "</table><br />";
	
		$force_login = get_option("allow_force_login");
		echo '<input name="force_login" type="checkbox" value="1"' .  ($force_login ? 'checked' : '') . '/> Force user login';

		echo "<div class=\"submit\"><input type=\"submit\" name=\"info_update\" value=\"" . __('Save', 'allow_option') . "\" /></div></form>";
	
		echo "</div>";
	
	}
	
	function allow_saveForm() {
		$len = count($_POST["user"]);
		if ($len){
			$opts = array();
			for ($i = 0; $i < $len; $i++) {
				if ($_POST["user"][$i] ){
					if (count($_POST["cat"][$i])>0){
						foreach($_POST["cat"][$i] as $ucat){
							$opts[$_POST["user"][$i]][] = (int)$ucat;
						}
					}
				}
			}
			update_option("allow_option", $opts);
		}

		$force_login = $_POST["force_login"];
		if (isset($force_login)) {
			update_option("allow_force_login", true);
		} else {
			update_option("allow_force_login", false);
		}
		return true;
	}

	function allow_menu() {
		add_management_page(__('Allow Categories'),
						 __('Allow Categories'),
						 10, basename(__FILE__), array('Allow_Category', 'allow_form'));
		
	}



}

add_option('allow_option', 		array(), '', false);
add_action('admin_menu', 		array('Allow_Category','allow_menu'));
add_action('admin_head', 		array('Allow_Category','allow_adminHead'));
if('wp-login.php' != $pagenow && 'wp-register.php' != $pagenow) add_action('template_redirect',		array('Allow_Category','allow_catRedirect'));
//add_action('wp',			array('Allow_Category','allow_parse'));
add_action('delete_post',		array('Allow_Category','allow_delPost'));
add_action('posts_where',		array('Allow_Category','allow_where'));
